@@ -1,8 +1,9 @@
 /*
  * virtual private network daemon (vpnd)
  *
- * cryptographic stuff (c) 1999 Andreas Steinmetz, astmail@yahoo.com
+ * cryptographic stuff (c) 1999 Andreas Steinmetz, ast@domdv.de
  * other code (c) 1999 D.O.M. Datenverarbeitung GmbH, author Andreas Steinmetz
+ * (c) 2001 Andreas Steinmetz
  *
  * License:
  * This code is in the public domain (*) under the GNU public license.
@@ -36,7 +37,9 @@
 
 int icmphandler(VPN *anchor)
 {
+#if !defined(SunOS)
 	int i;		/* used to set socket to nonblocking */
+#endif
 
 
 	/* debug message */
@@ -54,20 +57,33 @@ int icmphandler(VPN *anchor)
 
 	/* set socket to nonblocking, handle errors */
 
+#if defined(SunOS)
+	if(fcntl(anchor->ping,F_SETFL,O_NONBLOCK))
+		JUMP("fcntl(O_NONBLOCK)",err2);
+#else
 	i=1;
 	if(ioctl(anchor->ping,FIONBIO,&i))JUMP("ioctl(FIONBIO)",err2);
+#endif
 
 	/* fill icmp output structure with random data */
 
-	getrandom(anchor->icmp_out,sizeof(anchor->icmp_out),anchor);
+	getrandom(anchor->icmp_out,sizeof(anchor->icmp_out),0,anchor);
 
 	/* preset icmp header */
 
+#if defined(LINUX)
 	((struct icmphdr *)(anchor->icmp_out))->type=ICMP_ECHO;
 	((struct icmphdr *)(anchor->icmp_out))->code=0;
 	((struct icmphdr *)(anchor->icmp_out))->checksum=0;
 	((struct icmphdr *)(anchor->icmp_out))->un.echo.id=getpid();
 	((struct icmphdr *)(anchor->icmp_out))->un.echo.sequence=0;
+#elif defined(FreeBSD) || defined(OSNetBSD) || defined(OSOpenBSD) || defined(SunOS)
+	((struct icmp *)(anchor->icmp_out))->icmp_type=ICMP_ECHO;
+	((struct icmp *)(anchor->icmp_out))->icmp_code=0;
+	((struct icmp *)(anchor->icmp_out))->icmp_cksum=0;
+	((struct icmp *)(anchor->icmp_out))->icmp_hun.ih_idseq.icd_id=getpid();
+	((struct icmp *)(anchor->icmp_out))->icmp_hun.ih_idseq.icd_seq=0;
+#endif
 
 	/* prepare target address structure */
 
@@ -160,7 +176,11 @@ void sendicmp(VPN *anchor)
 
 	/* reset checksum in icmp header */
 
+#if defined(LINUX)
 	((struct icmphdr *)(anchor->icmp_out))->checksum=0;
+#elif defined(FreeBSD) || defined(OSNetBSD) || defined(OSOpenBSD) || defined(SunOS)
+	((struct icmp *)(anchor->icmp_out))->icmp_cksum=0;
+#endif
 
 	/* calculate packet checksum */
 
@@ -174,10 +194,17 @@ void sendicmp(VPN *anchor)
 
 	/* write checksum complement to header */
 
+#if defined(LINUX)
 	((WORD08 *)&(((struct icmphdr *)(anchor->icmp_out))->checksum))[0]=
 		(WORD08)(sum);
 	((WORD08 *)&(((struct icmphdr *)(anchor->icmp_out))->checksum))[1]=
 		(WORD08)(sum>>8);
+#elif defined(FreeBSD) || defined(OSNetBSD) || defined(OSOpenBSD) || defined(SunOS)
+	((WORD08 *)&(((struct icmp *)(anchor->icmp_out))->icmp_cksum))[0]=
+		(WORD08)(sum);
+	((WORD08 *)&(((struct icmp *)(anchor->icmp_out))->icmp_cksum))[1]=
+		(WORD08)(sum>>8);
+#endif
 
 	/* send echo request */
 
@@ -187,7 +214,11 @@ void sendicmp(VPN *anchor)
 
 	/* next sequence number */
 
+#if defined(LINUX)
 	((struct icmphdr *)(anchor->icmp_out))->un.echo.sequence+=1;
+#elif defined(FreeBSD) || defined(OSNetBSD) || defined(OSOpenBSD) || defined(SunOS)
+	((struct icmp *)(anchor->icmp_out))->icmp_hun.ih_idseq.icd_seq+=1;
+#endif
 
 	/* debug message */
 
