@@ -1,8 +1,9 @@
 /*
  * virtual private network daemon (vpnd)
  *
- * cryptographic stuff (c) 1999 Andreas Steinmetz, astmail@yahoo.com
+ * cryptographic stuff (c) 1999 Andreas Steinmetz, ast@domdv.de
  * other code (c) 1999 D.O.M. Datenverarbeitung GmbH, author Andreas Steinmetz
+ * (c) 2001,2005 Andreas Steinmetz
  *
  * License:
  * This code is in the public domain (*) under the GNU public license.
@@ -16,6 +17,7 @@
  * (*) 'public domain' is used here in the sense of the Wassenaar treaty.
  */
 
+#define SYSLOG_NAMES
 #include "vpnd.h"
 
 /*============================================================================*/
@@ -82,6 +84,12 @@ static CONFIG config[]=
 { "hmac",      1,1,2 },
 { "slipup",    1,1,1 },
 { "slipdown",  1,1,1 },
+{ "facility",  1,1,1 },
+{ "socks",     1,1,3 },
+{ "ppp",       1,0,0 },
+{ "control",   1,1,1 },
+{ "packetize", 1,0,0 },
+{ "smallrtp",  1,0,0 },
 { NULL,        0,0,0 }
 };
 
@@ -124,6 +132,12 @@ static CONFIG config[]=
 #define IDX_HMAC		43
 #define IDX_SLIPUP		44
 #define IDX_SLIPDOWN		45
+#define IDX_FACILITY		46
+#define IDX_SOCKS		47
+#define IDX_PPP			48
+#define IDX_CONTROL		49
+#define IDX_PACKETIZE		50
+#define IDX_SMALLRTP		51
 
 #define CFG_MODE(a)		anchor->config[IDX_MODE].a
 #define CFG_LOCAL(a)		anchor->config[IDX_LOCAL].a
@@ -165,6 +179,42 @@ static CONFIG config[]=
 #define CFG_HMAC(a)		anchor->config[IDX_HMAC].a
 #define CFG_SLIPUP(a)		anchor->config[IDX_SLIPUP].a
 #define CFG_SLIPDOWN(a)		anchor->config[IDX_SLIPDOWN].a
+#define CFG_FACILITY(a)		anchor->config[IDX_FACILITY].a
+#define CFG_SOCKS(a)		anchor->config[IDX_SOCKS].a
+#define CFG_CONTROL(a)		anchor->config[IDX_CONTROL].a
+
+/* Solaris syslog facilities */
+
+#ifdef SunOS
+#ifdef SYSLOG_NAMES
+static struct
+{
+	char *c_name;
+	int  c_val;
+} facilitynames[] =
+{
+	{ "kern", LOG_KERN },
+	{ "user", LOG_USER },
+	{ "mail", LOG_MAIL },
+	{ "daemon", LOG_DAEMON },
+	{ "auth", LOG_AUTH },
+	{ "syslog", LOG_SYSLOG },
+	{ "lpr", LOG_LPR },
+	{ "news", LOG_NEWS },
+	{ "uucp", LOG_UUCP },
+	{ "cron", LOG_CRON },
+	{ "local0", LOG_LOCAL0 },
+	{ "local1", LOG_LOCAL1 },
+	{ "local2", LOG_LOCAL2 },
+	{ "local3", LOG_LOCAL3 },
+	{ "local4", LOG_LOCAL4 },
+	{ "local5", LOG_LOCAL5 },
+	{ "local6", LOG_LOCAL6 },
+	{ "local7", LOG_LOCAL7 },
+	{ NULL, -1 }
+};
+#endif
+#endif
 
 /* numeric configuration data keep sequence! */
 
@@ -202,6 +252,10 @@ static NUMDATA numdata[]=
 	{ IDX_XFILTER,    -1, 1,               0               },
 	{ IDX_OOBFIX,     -1, 1,               0               },
 	{ IDX_HMAC,        1, 2,               0               },
+	{ -IDX_SOCKS,      1, 65535,           1080            },
+	{ IDX_PPP,        -1, 1,               0               },
+	{ IDX_PACKETIZE,  -1, 1,               0               },
+	{ IDX_SMALLRTP,   -1, 1,               0               },
 	{ 0,               0, 0,               0               }
 
 };
@@ -290,7 +344,11 @@ int checkfile(char *pathname,int perms)
 
 	if(stb.st_uid||stb.st_gid)
 	{
+#if defined(LINUX) || defined(SunOS)
 		printf("'%s' must be owned by root:root.\n",pathname);
+#elif defined(FreeBSD) || defined(OSNetBSD) || defined(OSOpenBSD)
+		printf("'%s' must be owned by root:wheel.\n",pathname);
+#endif
 		return 0;
 	}
 
@@ -299,7 +357,7 @@ int checkfile(char *pathname,int perms)
 	if(stb.st_mode&perms)
 	{
 		printf("'%s' permissions must be %03o or better.\n",
-			pathname,0777&perms);
+			pathname,0777^perms);
 		return 0;
 	}
 
@@ -700,6 +758,68 @@ int parse(VPN *anchor)
 	anchor->xfilter=(WORD08)(*u.w16++);
 	anchor->oobfix=(WORD08)(*u.w16++);
 	anchor->hmacopt=(int)(*u.w16++);
+	anchor->socksport=*u.w16++;
+	if(*u.w16++)
+	{
+		anchor->ifhandler=ppphandler;
+		anchor->iftype="ppp";
+		anchor->pkt_init=pkg_ppp_init;
+		anchor->if_to_pkt=ppp_to_pkg;
+		anchor->pkt_to_if=pkg_to_ppp;
+	}
+	anchor->pktmode=*u.w16++;
+	anchor->smallrtp=*u.w16++;
+
+	/* check control socket, if given */
+
+	if(CFG_CONTROL(found))
+	{
+		/* check pathname length */
+
+		if(strlen(CFG_CONTROL(p1))>=UNIX_PATH_MAX)
+		{
+			printf("Control socket pathname too long.\n");
+			return 0;
+		}
+
+		/* store socket path */
+
+		control=CFG_CONTROL(p1);
+	}
+
+	/* check socks if given */
+
+	if(CFG_SOCKS(found))
+	{
+		/* assert that socks is only used in TCP/IP mode */
+
+		if(anchor->serialmode)
+		{
+			printf("Socks cannot be used in serial line mode.\n");
+			return 0;
+		}
+
+		/* assert that socks is only used in client mode */
+
+		if(!anchor->mode)
+		{
+			printf("Socks can only be used in client mode.\n");
+			return 0;
+		}
+
+		/* assert that socks server ip is valid */
+
+		if(chkip(CFG_SOCKS(p1)))anchor->socksip=CFG_SOCKS(p1);
+		else
+		{
+			printf("Illegal socks server ip.\n");
+			return 0;
+		}
+
+		/* store socks username if given */
+
+		if(CFG_SOCKS(p3))anchor->socksuser=CFG_SOCKS(p3);
+	}
 
 	/* check all routes, if given */
 
@@ -739,6 +859,22 @@ int parse(VPN *anchor)
 		else anchor->hmacmode=1;
 	}
 
+	/* get syslog facility if given */
+
+	if(CFG_FACILITY(found))for(i=0;;i++)
+	{
+		if(!facilitynames[i].c_name)
+		{
+			printf("unknown syslog facility name.\n");
+			return 0;
+		}
+		if(!strcmp(facilitynames[i].c_name,CFG_FACILITY(p1)))
+		{
+			facility=facilitynames[i].c_val;
+			break;
+		}
+	}
+
 	/* get keyfile if given */
 
 	if(CFG_KEYFILE(found))anchor->keyfile=CFG_KEYFILE(p1);
@@ -749,7 +885,21 @@ int parse(VPN *anchor)
 
 	/* use optional random device file */
 
-	if(CFG_RANDOMDEV(found))anchor->randomdev=CFG_RANDOMDEV(p1);
+	if(CFG_RANDOMDEV(found))
+	{
+#ifdef SunOS
+		/* check pathname length */
+
+		if(strlen(CFG_RANDOMDEV(p1))>=UNIX_PATH_MAX)
+		{
+			printf("Random device socket pathname too long.\n");
+			return 0;
+		}
+#endif
+		/* store pathname */
+
+		anchor->randomdev=CFG_RANDOMDEV(p1);
+	}
 
 	/* set pathname of linkup process if requested */
 
@@ -875,6 +1025,8 @@ int parse(VPN *anchor)
 		anchor->cipher=NONE;
 		anchor->encrypt=(void (*)(WORD08 *,WORD32 *))(dummy);
 	}
+	if(anchor->pktmode&&anchor->mtu>MAXBLOCK)anchor->mtu=MAXBLOCK;
+	if(!anchor->pktmode)anchor->smallrtp=0;
 
 	/* debug message */
 
