@@ -1,8 +1,9 @@
 /*
  * virtual private network daemon (vpnd)
  *
- * cryptographic stuff (c) 1999 Andreas Steinmetz, astmail@yahoo.com
+ * cryptographic stuff (c) 1999 Andreas Steinmetz, ast@domdv.de
  * other code (c) 1999 D.O.M. Datenverarbeitung GmbH, author Andreas Steinmetz
+ * (c) 2001 Andreas Steinmetz
  *
  * License:
  * This code is in the public domain (*) under the GNU public license.
@@ -57,6 +58,9 @@
 
 int sliphandler(int smode,VPN *anchor)
 {
+#ifdef SunOS
+	return 3;
+#else
 	int i;			/* general purpose	*/
 	int j;			/* general purpose	*/
 	int k;			/* general purpose	*/
@@ -64,6 +68,9 @@ int sliphandler(int smode,VPN *anchor)
 	int err;		/* return code		*/
 	int quiet;		/* >0 for quiet errors	*/
 	struct ifreq ifr;	/* used to set up if	*/
+#if defined(FreeBSD) || defined(OSNetBSD) || defined(OSOpenBSD)
+	struct ifaliasreq ifa;	/* used to set up if	*/
+#endif
 	struct termios t;	/* pty/tty settings	*/
 
 
@@ -170,15 +177,25 @@ int sliphandler(int smode,VPN *anchor)
 
 	/* set line discipline to SLIP, react to errors */
 
+#ifdef LINUX
 	i=N_SLIP;
 	if((anchor->proxy=ioctl(anchor->tty,TIOCSETD,&i))<0)
 		JUMP("ioctl(TIOCSETD)",err3);
+#elif defined(FreeBSD) || defined(OSNetBSD) || defined(OSOpenBSD)
+	i=SLIPDISC;
+	if(ioctl(anchor->tty,TIOCSETD,&i)<0)
+		JUMP("ioctl(TIOCSETD)",err3);
+	if(ioctl(anchor->tty,SLIOCGUNIT,&(anchor->proxy))<0)
+		JUMP("ioctl(SLIOCGUNIT)",err3);
+#endif
 
 	/* set encapsulation, react to errors */
 
+#ifdef LINUX
 	i=anchor->cslip?SL_MODE_CSLIP:SL_MODE_SLIP;
 	if(ioctl(anchor->tty,SIOCSIFENCAP,&i)<0)
 		JUMP("ioctl(SIOCSIFENCAP)",err4);
+#endif
 
 	/* get control socket, react to errors */
 
@@ -190,38 +207,74 @@ int sliphandler(int smode,VPN *anchor)
 
 	/* create interface name */
 
+	memset(&ifr,0,sizeof(ifr));
 	sprintf(ifr.ifr_name,"sl%u",anchor->proxy);
+#if defined(FreeBSD) || defined(OSNetBSD) || defined(OSOpenBSD)
+	memset(&ifa,0,sizeof(ifa));
+	sprintf(ifa.ifra_name,"sl%u",anchor->proxy);
+#endif
 
 	/* set up interface address, react to errors */
 
+#ifdef LINUX
 	((struct sockaddr_in *)(&(ifr.ifr_addr)))->sin_family=AF_INET;
 	((struct sockaddr_in *)(&(ifr.ifr_addr)))->sin_addr.s_addr=
 		getaddr(anchor->local_ip);
 	if(ioctl(i,SIOCSIFADDR,&ifr)<0)j++;
+#elif defined(FreeBSD) || defined(OSNetBSD) || defined(OSOpenBSD)
+	((struct sockaddr_in *)(&(ifa.ifra_addr)))->sin_len=
+		sizeof(struct sockaddr_in);
+	((struct sockaddr_in *)(&(ifa.ifra_addr)))->sin_family=AF_INET;
+	((struct sockaddr_in *)(&(ifa.ifra_addr)))->sin_addr.s_addr=
+		getaddr(anchor->local_ip);
+#endif
 
 	/* set up interface peer address, react to errors */
 
+#ifdef LINUX
 	((struct sockaddr_in *)(&(ifr.ifr_dstaddr)))->sin_family=AF_INET;
 	((struct sockaddr_in *)(&(ifr.ifr_dstaddr)))->sin_addr.s_addr=
 		getaddr(anchor->remote_ip);
 	if(!j)if(ioctl(i,SIOCSIFDSTADDR,&ifr)<0)j++;
+#elif defined(FreeBSD) || defined(OSNetBSD) || defined(OSOpenBSD)
+	((struct sockaddr_in *)(&(ifa.ifra_broadaddr)))->sin_len=
+		sizeof(struct sockaddr_in);
+	((struct sockaddr_in *)(&(ifa.ifra_broadaddr)))->sin_family=AF_INET;
+	((struct sockaddr_in *)(&(ifa.ifra_broadaddr)))->sin_addr.s_addr=
+		getaddr(anchor->remote_ip);
+#endif
 
 	/* set up interface netmask, react to errors */
 
+#ifdef LINUX
 	((struct sockaddr_in *)(&(ifr.ifr_netmask)))->sin_family=AF_INET;
 	((struct sockaddr_in *)(&(ifr.ifr_netmask)))->sin_addr.s_addr=
 		htonl(0xffffffff);
 	if(!j)if(ioctl(i,SIOCSIFNETMASK,&ifr)<0)j++;
+#elif defined(FreeBSD) || defined(OSNetBSD) || defined(OSOpenBSD)
+	((struct sockaddr_in *)(&(ifa.ifra_mask)))->sin_len=
+		sizeof(struct sockaddr_in);
+	((struct sockaddr_in *)(&(ifa.ifra_mask)))->sin_addr.s_addr=
+		htonl(0xffffffff);
+	if(!j)if(ioctl(i,SIOCAIFADDR,&ifa)<0)j++;
+#endif
 
 	/* set up interface mtu, react to errors */
 
+#ifndef OSOpenBSD
 	ifr.ifr_mtu=anchor->mtu;
 	if(!j)if(ioctl(i,SIOCSIFMTU,&ifr)<0)j++;
+#endif
 
 	/* activate interface, react to errors */
 
+#ifdef LINUX
 	ifr.ifr_flags=IFF_UP|IFF_RUNNING|IFF_POINTOPOINT|IFF_NOARP|
 		IFF_MULTICAST;
+#elif defined(FreeBSD) || defined(OSNetBSD) || defined(OSOpenBSD)
+	ifr.ifr_flags=IFF_UP|IFF_POINTOPOINT|IFF_NOARP|IFF_MULTICAST;
+	if(anchor->cslip)ifr.ifr_flags|=SC_COMPRESS;
+#endif
 	if(!j)if(ioctl(i,SIOCSIFFLAGS,&ifr)<0)j++;
 
 	/* close control socket */
@@ -230,16 +283,22 @@ int sliphandler(int smode,VPN *anchor)
 
 	/* signal any errors encountered */
 
+#ifdef LINUX
 	if(j)JUMP("ioctl(SIOCSIFADDR,SIOCSIFDSTADDR,SIOCSIFNETMASK," \
 		"SIOCSIFMTU,SIOCSIFFLAGS)",err4);
+#elif defined(FreeBSD) || defined(OSNetBSD) || defined(OSOpenBSD)
+	if(j)JUMP("ioctl(SIOCAIFADDR,SIOCSIFMTU,SIOCSIFFLAGS)",err4);
+#endif
 
 	/* set up route to partner, react to errors */
 
+#ifdef LINUX
 	if(!anchor->autoroute)
 	{
 		if(route(1,getaddr(anchor->remote_ip),-1,0,ifr.ifr_name))
 			GOTO("autoroute",err5);
 	}
+#endif
 
 	/* set up priority route to peer, react to errors */
 
@@ -277,7 +336,12 @@ delete:	if(!anchor->slip)GOTO("slip link not up",err0);
 
 	/* create interface name */
 
+	memset(&ifr,0,sizeof(ifr));
 	sprintf(ifr.ifr_name,"sl%u",anchor->proxy);
+#if defined(FreeBSD) || defined(OSNetBSD) || defined(OSOpenBSD)
+	memset(&ifa,0,sizeof(ifa));
+	sprintf(ifa.ifra_name,"sl%u",anchor->proxy);
+#endif
 
 	/* execute slip down process, if there is any */
 
@@ -313,6 +377,7 @@ err6:	if(quiet)err++;
 
 	/* delete route to partner, react to errors */
 
+#ifdef LINUX
 	if(!anchor->autoroute)
 	{
 		if(route(0,getaddr(anchor->remote_ip),-1,0,ifr.ifr_name))
@@ -320,6 +385,9 @@ err6:	if(quiet)err++;
 	}
 
 err5:	if(quiet)err++;
+#elif defined(FreeBSD) || defined(OSNetBSD) || defined(OSOpenBSD)
+	if(quiet)err++;
+#endif
 
 	/* get control socket, react to errors */
 
@@ -334,16 +402,39 @@ err5:	if(quiet)err++;
 	{
 		/* deactivate interface, react to errors */
 
+#if defined(FreeBSD) || defined(OSNetBSD) || defined(OSOpenBSD)
+		((struct sockaddr_in *)(&(ifa.ifra_addr)))->sin_len=
+			sizeof(struct sockaddr_in);
+		((struct sockaddr_in *)(&(ifa.ifra_addr)))
+			->sin_family=AF_INET;
+		((struct sockaddr_in *)(&(ifa.ifra_addr)))
+			->sin_addr.s_addr=getaddr(anchor->local_ip);
+		((struct sockaddr_in *)(&(ifa.ifra_broadaddr)))->sin_len=
+			sizeof(struct sockaddr_in);
+		((struct sockaddr_in *)(&(ifa.ifra_broadaddr)))
+			->sin_family=AF_INET;
+		((struct sockaddr_in *)(&(ifa.ifra_broadaddr)))
+			->sin_addr.s_addr=getaddr(anchor->remote_ip);
+		((struct sockaddr_in *)(&(ifa.ifra_mask)))->sin_len=
+			sizeof(struct sockaddr_in);
+		((struct sockaddr_in *)(&(ifa.ifra_mask)))
+			->sin_addr.s_addr=htonl(0xffffffff);
+		if(ioctl(i,SIOCDIFADDR,&ifa)<0)
+			ERRNO("ioctl(SIOCDIFADDR)");
+#endif
+
 		ifr.ifr_flags=0;
 		if(ioctl(i,SIOCSIFFLAGS,&ifr)<0)if(!quiet++)
 			ERRNO("ioctl(SIOCSIFFLAGS)");
 
 		/* reset interface address, react to errors */
 
+#ifdef LINUX
 		((struct sockaddr_in *)(&(ifr.ifr_addr)))->sin_family=AF_INET;
 		((struct sockaddr_in *)(&(ifr.ifr_addr)))->sin_addr.s_addr=0;
 		if(ioctl(i,SIOCSIFADDR,&ifr)<0)if(!quiet++)
 			ERRNO("ioctl(SIOCSIFADDR)");
+#endif
 
 		/* close control socket */
  
@@ -387,4 +478,5 @@ err0:	LEAVE("sliphandler");
 	/* signal success or error */
 
 	return err;
+#endif
 }
